# Copyright 2016 GetWellNetwork, Inc., BSD copyright and disclaimer apply

import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GObject
import json
import os
from random import randint
import sys

from bjarkan import find_adapter, find_device, find_adapter_in_objects, find_device_in_objects, get_managed_objects
import bjarkan.list_devices



DBusGMainLoop( set_as_default=True )



class Agent( dbus.service.Object ):
	"""This is the code that deals with generating a PIN code if needed by the bluetooth device
	we are connecting to in order to complete the authentication handshake.
	"""
	AGENT_INTERFACE = 'org.bluez.Agent1'
	exit_on_release = True


	def set_exit_on_release( self, exit_on_release ):
		self.exit_on_release = exit_on_release


	@dbus.service.method( AGENT_INTERFACE, in_signature = '', out_signature = '' )
	def Release( self ):
		if self.exit_on_release:
			mainloop.quit()


	@dbus.service.method( AGENT_INTERFACE, in_signature = 'os', out_signature = '' )
	def DisplayPinCode( self, device, pincode ):
		print( 'DisplayPinCode ({}, {})'.format( device, pincode ) )


	@dbus.service.method( AGENT_INTERFACE, in_signature = 'ou', out_signature = '' )
	def RequestConfirmation( self, device, passkey ):
		print( 'RequestConfirmation ({}, {})'.format( device, passkey ) )


def pair_device( device ):
	"""Does the act of attempting to pair to the bluetooth device specified.

	Args:
		device: mac address of the bluetooth device to connect to
		json: to change the return format to json instead of plain text. Meant for API use.
	"""
	result = None
	code = None
	results = {}
	mainloop = GObject.MainLoop()
	bus = dbus.SystemBus()
	capability = 'KeyboardDisplay'
	path = '/test/agent'
	agent = Agent( bus, path )
	obj = bus.get_object( 'org.bluez', '/org/bluez' )
	manager = dbus.Interface( obj, 'org.bluez.AgentManager1')
	dev = find_device( device )


	def pair_reply():
		result = 'Success'
		dev_path = dev.object_path
		props = dbus.Interface(bus.get_object( 'org.bluez', dev_path ), 'org.freedesktop.DBus.Properties' )

		props.Set( DEVICE_INTERFACE, 'Trusted', True )
		dev.Connect()
		mainloop.quit()
		results.update( { 'result': result, 'code': code } )


	def pair_error( error ):
		result = 'Error'
		err = error.get_dbus_name()
		if err == 'org.freedesktop.DBus.Error.NoReply' and dev:
			code = 'Timeout'
			dev.CancelPairing()
		if err in ( 'org.bluez.Error.AuthenticationCanceled', 'org.bluez.Error.AuthenticationFailed',
				'org.bluez.Error.AuthenticationRejected', 'org.bluez.Error.AuthenticationTimeout' ):
			code = 'AuthenticationError'
		else:
			code = 'CreatingDeviceFailed'

		mainloop.quit()
		results.update( { 'result': result, 'code': code } )


	manager.RegisterAgent( path, capability )
	agent.set_exit_on_release( False )
	dev.Pair( reply_handler = pair_reply, error_handler = pair_error, timeout = 60000 )
	mainloop.run()
	return results



def unpair_device( device ):
	"""Does the act of attempting to unpair to the bluetooth device specified.

	Args:
		device: mac address of the bluetooth device to connect to
		json: to change the return format to json instead of plain text. Meant for API use.
	"""
	result = None
	code = None
	managed_objects = get_managed_objects()
	adapter = find_adapter_in_objects( managed_objects )
	dev = find_device_in_objects( managed_objects, device )
	dev_path = dev.object_path
	try:
		adapter.RemoveDevice( dev_path )
		result = 'Success'
	except dbus.exceptions.DBusException as e:
		result = 'Error'
		code = e.get_dbus_name()
	except:
		result = 'Error'
		code = 'UnpairFailure'
	finally:
		return { 'result': result, 'code': code }



def disconnect_device( device ):
	"""Does the act of attempting to discconnect to the bluetooth device specified.

	Args:
		device: mac address of the bluetooth device to connect to
		json: to change the return format to json instead of plain text. Meant for API use.
	"""
	result = None
	code = None
	dev = find_device( device )
	try:
		dev.Disconnect()
		result = 'Success'
	except dbus.exceptions.DBusException as e:
		result = 'Error'
		code = e.get_dbus_name()
	except:
		result = 'Error'
		code = 'DisconnectFailure'
	finally:
		return { 'result': result, 'code': code }



def connect_device( device ):
	"""Does the act of attempting to connect to the bluetooth device specified.

	Args:
		device: mac address of the bluetooth device to connect to
		json: to change the return format to json instead of plain text. Meant for API use.
	"""
	result = None
	code = None
	dev = find_device( device )

	try:
		dev.Connect()
		result = 'Success'
	except dbus.exceptions.DBusException as e:
		result = 'Error'
		code = e.get_dbus_name()
	except:
		result = 'Error'
		code = 'ConnectionFailure'
	finally:
		return { 'result': result, 'code': code }



#!/usr/bin/python3

# Copyright 2016 GetWellNetwork, Inc., BSD copyright and disclaimer apply

from argparse import ArgumentParser
import json
import sys
from bjarkan.connection_manager import *
import bjarkan.list_devices



def format_device_data( devices, format_json ):
	if format_json:
		data = []
		for device in devices:
			device_data = {}
			device_data['address'] = device['address']
			device_data['rssi'] = device['rssi']
			device_data['paired'] = device['paired']
			device_data['connected'] = device['connected']
			device_data['alias'] = device['alias']
			data.append( device_data )

		print( json.dumps( data ) )
	else:
		for device in devices:
			print(
				'{!s} {!s} {!s} {!s} {!s}'.format(
					device['address'],
					device['rssi'],
					device['paired'],
					device['connected'],
					device['alias']
				)
			)



def format_results( results, format_json ):
	if format_json:
		print( json.dumps( { 'result': results['result'], 'code': results['code'] } ) )
	else:
		print( 'result: {}, code: {}'.format( results['result'], results['code'] ) )



def pair( args ):
	return format_results( pair_device( args.device ), args.json )



def unpair( args ):
	return format_results( unpair_device( args.device ), args.json )



def connect( args ):
	return format_results( connect_device( args.device ), args.json )



def disconnect( args ):
	return format_results( disconnect_device( args.device ), args.json )



def connected( args ):
	return format_device_data( bjarkan.list_devices.connected_devices(), args.json )



def paired( args ):
	return format_device_data( bjarkan.list_devices.paired_devices(), args.json )



def scan( args ):
	return format_device_data( bjarkan.list_devices.all_devices(), args.json )



def main():
	parser = ArgumentParser( description = 'Connect to specifed BT device' )
	parser.add_argument( '-j', '--json', action = 'store_true', help = 'Change output format to json instead of plain text' )
	subparsers = parser.add_subparsers( metavar = 'COMMAND' )
	subparsers.required = True

	pair_parser = subparsers.add_parser( 'pair', help = 'Pair a device (pairing will also connect)' )
	pair_parser.add_argument( '-d', '--device', required = True, help = 'Specify the device to pair' )
	pair_parser.set_defaults( func = pair )

	unpair_parser = subparsers.add_parser( 'unpair', help = 'Unpair a device' )
	unpair_parser.add_argument( '-d', '--device', required = True, help = 'Specify the device to unpair' )
	unpair_parser.set_defaults( func = unpair )

	connect_parser = subparsers.add_parser( 'connect', help = 'Connect a new device' )
	connect_parser.add_argument( '-d', '--device', required = True, help = 'Specify the device to connect to' )
	connect_parser.set_defaults( func = connect )

	disconnect_parser = subparsers.add_parser( 'disconnect', help = 'Disconnect a device' )
	disconnect_parser.add_argument( '-d', '--device', required = True, help = 'Specify the device to disconnect from' )
	disconnect_parser.set_defaults( func = disconnect )

	paired_parser = subparsers.add_parser( 'paired-devices', help = 'Show all paired devices' )
	paired_parser.set_defaults( func = paired )

	connected_parser = subparsers.add_parser( 'connected-devices', help = 'Show all connected devices' )
	connected_parser.set_defaults( func = connected )

	list_parser = subparsers.add_parser( 'scan', help = 'Show all currently known devices' )
	list_parser.set_defaults( func = scan )

	args = parser.parse_args()

	result = args.func( args )
	if result:
		return result

	return 0



if __name__ == '__main__':
    main()
